@@ -10,6 +10,9 @@
 
 #include "super.h"
 #include "version.h"
+#ifdef __linux__
+#include <shadow.h>
+#endif
 
 #ifdef USE_NETGROUP
 #define netgrp_u_compare(pattern, user) innetgr(pattern, NULL, user, NULL)
@@ -248,7 +251,7 @@ int
 get_encrypted_pw()
 {
     struct passwd *caller_pw;
-    struct spwd *caller_pw;
+    struct spwd *caller_sw;
 
     if (!(caller_pw = getpwnam(ui.caller)))
 	return Error(0, 0, "No password entry for user %s.\n", ui.caller);
@@ -258,12 +261,12 @@ get_encrypted_pw()
      */
     if (strcmp(caller_pw->sp_pwdp, "x") == 0) {
 	/* Shadow password in use... */
-	if (!(caller_pw = getspnam(ui.caller)))
+	if (!(caller_sw = getspnam(ui.caller)))
 	    return Error(1, 0,
 	    "Failed to obtain shadow password entry for user %s: ",
 	    ui.caller);
-	strcpy(ui.encr, caller_pw->sp_pwdp);
-	strncpy(ui.salt, caller_pw->sp_pwdp, 2);
+	strcpy(ui.encr, caller_sw->sp_pwdp);
+	strncpy(ui.salt, caller_sw->sp_pwdp, 2);
 	ui.salt[2] = '\0';
 
     } else {
@@ -282,10 +285,18 @@ get_encrypted_pw()
 {
     /* Vanilla password file lookup */
     struct passwd *caller_pw;
+#ifdef __linux__
+    struct  spwd    *spwd = 0L;
+#endif
 
     if (!(caller_pw = getpwnam(ui.caller)))
 	return Error(0, 0, "No password entry for user %s.\n", ui.caller);
 
+#ifdef __linux__
+    if ((spwd = getspnam(ui.caller)))
+        caller_pw->pw_passwd = spwd->sp_pwdp;
+#endif
+
     strcpy(ui.encr, caller_pw->pw_passwd);
     strncpy(ui.salt, caller_pw->pw_passwd, 2);
     ui.salt[2] = '\0';
